package edu.prlab.tyler.iotgateway.contract.common;

import edu.prlab.tyler.iotgateway.contract.DeviceContract;
import org.web3j.protocol.http.HttpService;
import org.web3j.quorum.Quorum;
import org.web3j.tx.ClientTransactionManager;
import org.web3j.tx.gas.ContractGasProvider;
import org.web3j.tx.gas.DefaultGasProvider;
import org.web3j.tx.gas.StaticGasProvider;

import java.math.BigInteger;
import java.util.Optional;

public abstract class QuorumContractService<Data, Contract extends org.web3j.tx.Contract> {
    private static final String rpcUrl = "http://localhost:22000";

    private static final ContractGasProvider DEFAULT_GAS_PROVIDER = new StaticGasProvider(
            BigInteger.ZERO,
            DefaultGasProvider.GAS_LIMIT
    );

    Quorum quorum;
    ContractGasProvider provider;
    ClientTransactionManager manager;

    public QuorumContractService(Quorum quorum, ContractGasProvider provider, ClientTransactionManager manager) {
        this.quorum = quorum;
        this.provider = provider;
        this.manager = manager;
    }

    //部屬合約
    protected Optional<Contract> deploy() {
        return Optional.ofNullable(Quorum.build(new HttpService(rpcUrl)))
                .flatMap(this::buildManagerAndMap)
                .map(this::deployContract)
                .filter(QuorumContractUtil::isContractValid);
    }

    //以帳戶第0個建立TransactionManager
    protected Optional<QuorumContractReference> buildManagerAndMap(Quorum quorum) {
        try {
            return Optional.ofNullable(new ClientTransactionManager(quorum, quorum.ethAccounts().send().getAccounts().get(0)))
                    .map(this::prepare);
        } catch (Exception e) {
            System.out.println("Cannot find quorum manager");
        }
        return Optional.empty();
    }

    //建立合約所需要的參數
    protected QuorumContractReference prepare(ClientTransactionManager manager) {
        return QuorumContractReference.builder()
                .quorum(quorum)
                .provider(DEFAULT_GAS_PROVIDER)
                .manager(manager)
                .build();
    }

    //部屬合約的子類別實作
    protected Contract deployContract(QuorumContractReference reference) {
        return null;
    }

    //載入合約
    protected Contract loadContract(QuorumContractReference reference) {
        return null;
    }

    //載入該合約資料
    protected Data loadData(Contract deviceContract) {
        return null;
    }

}
