package edu.prlab.tyler.iotgateway.gateway.services;

import edu.prlab.tyler.iotgateway.contract.DeviceContract;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractReference;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractService;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractUtil;
import edu.prlab.tyler.iotgateway.gateway.model.DeviceContractInfoMessage;
import org.web3j.quorum.Quorum;
import org.web3j.tx.ClientTransactionManager;
import org.web3j.tx.gas.ContractGasProvider;

public class DeviceContractService extends QuorumContractService<DeviceContractInfoMessage, DeviceContract> {

    public DeviceContractService(Quorum quorum, ContractGasProvider provider, ClientTransactionManager manager) {
        super(quorum, provider, manager);
    }



    @Override
    protected DeviceContract loadContract(QuorumContractReference reference) {
        return DeviceContract.load(reference.getAddress(), reference.getQuorum(), reference.getManager(), reference.getProvider());
    }

    @Override
    protected DeviceContractInfoMessage loadData(DeviceContract deviceContract) {
        return DeviceContractInfoMessage.builder()
                .ownerAddress(QuorumContractUtil.call(deviceContract.owner()))
                .deviceInfo(QuorumContractUtil.call(deviceContract.deviceInfo()))
                .privacyPolicy(QuorumContractUtil.call(deviceContract.privacypolicy()))
                .bindAddress(QuorumContractUtil.call(deviceContract.binded()))
                .build();
    }

    @Override
    protected DeviceContract deployContract(QuorumContractReference reference) {
        try {
            return DeviceContract.deploy(reference.getQuorum(), reference.getManager(), reference.getProvider()).send();
        } catch (Exception e) {
            System.out.println("Cannot deploy device contract.");
        }
        return null;
    }
}
