package edu.prlab.tyler.iotgateway.gateway.services;

import edu.prlab.tyler.iotgateway.contract.GatewayContract;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractReference;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractService;
import edu.prlab.tyler.iotgateway.contract.common.QuorumContractUtil;
import edu.prlab.tyler.iotgateway.gateway.model.GatewayContractInfoMessage;
import org.web3j.quorum.Quorum;
import org.web3j.tx.ClientTransactionManager;
import org.web3j.tx.gas.ContractGasProvider;

import java.math.BigInteger;
import java.util.stream.IntStream;

public class GatewayContractService extends QuorumContractService<GatewayContractInfoMessage, GatewayContract> {

    public GatewayContractService(Quorum quorum, ContractGasProvider provider, ClientTransactionManager manager) {
        super(quorum, provider, manager);
    }

    @Override
    protected GatewayContract loadContract(QuorumContractReference reference) {
        return GatewayContract.load(reference.getAddress(), reference.getQuorum(), reference.getManager(), reference.getProvider());
    }

    @Override
    protected GatewayContractInfoMessage loadData(GatewayContract gatewayContract) {
        BigInteger numDevices = QuorumContractUtil.call(gatewayContract.numDevices());
        return GatewayContractInfoMessage.builder()
                .numDevices(numDevices)
                .devices(() -> IntStream.range(0, numDevices.intValue())
                        .mapToObj(i -> QuorumContractUtil.call(gatewayContract.getDeviceAddress(BigInteger.valueOf(i))))
                        .iterator())
                .build();
    }

    @Override
    protected GatewayContract deployContract(QuorumContractReference reference) {
        try {
            return GatewayContract.deploy(reference.getQuorum(), reference.getManager(), reference.getProvider()).send();
        } catch (Exception e) {
            System.out.println("Cannot deploy gateway contract.");
        }
        return null;
    }
}
